﻿# Copyright (c) Microsoft Corporation. All rights reserved.
# Common.psm1 5.11.251.0 2016-11-04 10:37:08

# AzureUX-Katal dev-tp2-sqlrp retail-amd64

#requires -Version 4.0
#requires -Module AzureRM.Profile
#requires -Module AzureRM.Resources
#requires -Module AzureRM.AzureStackAdmin

<#
.SYNOPSIS
    Initializes the Azure PowerShell environment for use with an Azure Stack Environment.
#>
#Import-Module AzureRM.Profile
Import-Module -Name AzureRM -RequiredVersion 1.2.10
#Import-Module -Name "$PSScriptRoot\..\..\Telemetry\Telemetry.dll" -ErrorAction Stop -Verbose:$false # Telemetry

function Get-AzureStackEnvironment
{
    param(
        [Parameter(Mandatory=$true, HelpMessage="Azure Active directory Name")]
        [string]$AadTenantDirectoryName,
        [Parameter(Mandatory=$false, HelpMessage="Friendly name of Azure stack")]
        [string]$EnvironmentName = "AzureStack",
        [Parameter(Mandatory=$true, HelpMessage="Azure Stack Admin ARM Endpoint")]
        [string]$AdminArmEndpoint
    )
    $azEnvironment = Get-AzureRmEnvironment $EnvironmentName -ErrorAction SilentlyContinue
    if($azEnvironment -ne $null)
    {
       Remove-AzureRmEnvironment -Name $EnvironmentName -Force | Out-Null
    }
    #$domainDNSRoot = [System.Net.NetworkInformation.IPGlobalProperties]::GetIPGlobalProperties().DomainName
    $armEndpoint = $AdminArmEndpoint
    $endpoints = Invoke-RestMethod "${armEndpoint}/metadata/endpoints?api-version=2015-01-01";

    $azEnvironment = Add-AzureRmEnvironment -Name $EnvironmentName `
        -ActiveDirectoryEndpoint ("https://login.windows.net/$AadTenantDirectoryName/") `
        -ActiveDirectoryServiceEndpointResourceId $endpoints.authentication.audiences[0] `
        -ResourceManagerEndpoint ($armEndpoint) `
        -GalleryEndpoint $endpoints.galleryEndpoint `
        -GraphEndpoint $endpoints.graphEndpoint `
		-GraphAudience $endpoints.graphEndpoint
    $azEnvironment = Get-AzureRmEnvironment $EnvironmentName
    return $azEnvironment
}

function Add-AzureStackAccount
{
    param(
        [Parameter(Mandatory=$true, HelpMessage="Credential object of Azure Stack")]
        [PSCredential]$azCredential,
        [Parameter(Mandatory=$true, HelpMessage="Azure environment Object")]
        [object]$azEnvironment
    )
    if (-not $azCredential)
    {
        $azAccount = Add-AzureRmAccount -Environment $azEnvironment # Prompts for credentials.
    }
    else
    {
        $azAccount = Add-AzureRmAccount -Environment $azEnvironment -Credential $azCredential
    }
    $azAccount
}

function Initialize-AzureStackEnvironment
{
    param(
        [Parameter(Mandatory=$true, HelpMessage="Azure Active directory Name")]
        [string]$AadTenantDirectoryName,
        [Parameter(Mandatory=$false, HelpMessage="Friendly name of Azure stack")]
        [string]$EnvironmentName = "AzureStack",
        [Parameter(Mandatory=$true, HelpMessage="Credential object of Azure Stack")]
        [PSCredential]$azCredential,
		[Parameter(Mandatory=$true, HelpMessage="Azure Stack Admin ARM Endpoint URL")]
		[string]$AdminArmEndpoint
    )
    $azEnvironment = Get-AzureStackEnvironment -AadTenantDirectoryName $AadTenantDirectoryName -EnvironmentName $EnvironmentName -AdminArmEndpoint $AdminArmEndpoint
    $azAccount = Add-AzureStackAccount -azCredential $azCredential -azEnvironment $azEnvironment
    $azAccount
}

function Execute-Command 
{
    param(
        [Parameter(Mandatory=$true, HelpMessage="Command to be run")]
        [string]$Command,
        [Parameter(Mandatory=$true, HelpMessage="Parameters for the command")]
        [hashtable]$Parameters,
        [Parameter(Mandatory=$false, HelpMessage="Max Retry Count")]
        [int]$MaxRetryCount = 3,
        [Parameter(Mandatory=$false, HelpMessage="Retry duration")]
        [int]$RetryDuration = 300
    )
    $currentRetry = 0;
    $success = $false;
    do 
    {
        try 
        { 
            Write-Verbose -Message ("Started to execute [$Command] at {0} time(s) " -f ($currentRetry + 1))
            if($Parameters -ne $null -and $Parameters.Count -gt 0)
            {
                & $Command @parameters
            }
            else
            {
                & $Command
            }
            $success = $true;
            Write-Verbose -Message ("Completed execution of [$Command] at {0} time(s) " -f ($currentRetry + 1))
        } 
        catch [System.Exception] 
        {
            Write-Verbose -Message ("Failed to execute [$Command] at {0} time(s) " -f ($currentRetry + 1))
            if ($currentRetry -ge $MaxRetryCount) 
            {
                $message = "Can not execute [$Command] command. The error: " + $_.Exception.ToString();
                throw $message;
            } 
            else
            {
                $message = ("Can not execute [$Command] command at {0} time(s). Will retry. The error: " -f ($currentRetry + 1)) + $_.Exception.ToString();
                Write-Warning -Message $message
                Start-Sleep -s $RetryDuration;
            }
            $currentRetry++;
        }

    } while (!$success);
} 

function Test-AadCredential
{
    param(
        [Parameter(Mandatory=$true, HelpMessage="Azure Active directory Name")]
        [string]$AadTenantDirectoryName,
        [Parameter(Mandatory=$true, HelpMessage="Credential object of Azure Stack")]
        [PSCredential]$azCredential = "AzureStack",
		[Parameter(Mandatory=$true)]
		[string]$AdminArmEndpoint
		)
    $success = $false
    try
    {
        $azAccount = Initialize-AzureStackEnvironment -AadTenantDirectoryName $AadTenantDirectoryName -azCredential $azCredential -AdminArmEndpoint $AdminArmEndpoint
        $success =$true
    }
    catch
    {
        Write-Verbose -Message ("Cannot login Aad with account: {0}" -f $azCredential.UserName) -Verbose
        Write-Verbose -Message ("Errors are " + $_.ToString()) -Verbose
        $success =$false
    }
    return $success
}

function Test-NamespaceExistance
{
    param(
        [Parameter(Mandatory=$true, HelpMessage="Azure Active directory Name")]
        [string]$AadTenantDirectoryName,
        [Parameter(Mandatory=$true, HelpMessage="Credential object of Azure Stack")]
        [PSCredential]$azCredential,
        [Parameter(Mandatory=$true, HelpMessage="Namespace of the Azure provider")]
        [string]$namespace,
		[Parameter(Mandatory=$true, HelpMessage="Azure Stack Admin ARM Endpoint URL")]
		[string]$AdminArmEndpoint
    )
    $azAccount = Initialize-AzureStackEnvironment -AadTenantDirectoryName $AadTenantDirectoryName -azCredential $azCredential -AdminArmEndpoint $AdminArmEndpoint
    $rpNamespace = Get-AzureRMResourceProvider -ListAvailable | ? {$_.ProviderNamespace -eq $namespace}
    return $rpNamespace -eq 0
}

function Trace-DeploymentInternal
{
    param(
        [Parameter(Mandatory=$true, HelpMessage="Deployment GUID")]
        [string]$DeploymentGUID,
        [Parameter(Mandatory=$true, HelpMessage="Start time")]
        [string]$StartTime,
        [Parameter(Mandatory=$false, HelpMessage="End time")]
        [string]$EndTime = [System.DateTimeOffset]::Now,
        [Parameter(Mandatory=$true, HelpMessage="Status")]
        [string]$Status,
        [Parameter(Mandatory=$true, HelpMessage="Step")]
        [string]$Step,
        [Parameter(Mandatory=$true, HelpMessage="Filter to select files from path")]
        [string]$LogMessage,
        [Parameter(Mandatory=$false, HelpMessage="Trace level")]
        $TraceLevel = 'Info'
    )
    #Trace-Deployment -DeploymentGUID $deploymentGUID -Type "ARM" -RoleName "BillingProvider" -Step $step `
    #    -TraceLevel $TraceLevel -StartTime $StartTime -EndTime $EndTime -Status $Status -LogMessage $LogMessage 
}


function Upload-FilesToBlobStorage
{
    param(
        [Parameter(Mandatory=$true, HelpMessage="Path of files to upload")]
        [string]$path,
        [Parameter(Mandatory=$true, HelpMessage="Resource group name")]
        [string]$resourceGroupName,
        [Parameter(Mandatory=$true, HelpMessage="Storage account name")]
        [string]$storageAccountName,
        [Parameter(Mandatory=$true, HelpMessage="Storage container name")]
        [string]$storageContainerName,
        [Parameter(Mandatory=$false, HelpMessage="Filter to select files from path")]
        [string]$filter="*"
    )

    $blobs = @()

    # Resource group
    $resourceGroup = New-AzureRmResourceGroup -Name $resourceGroupName -Location "local" -Force
    Write-Verbose -Message "Resource Group: $($resourceGroup.ResourceGroupName)" -Verbose

    # Storage account
    $storageAccount = New-AzureRmStorageAccount -ResourceGroupName $resourceGroupName -StorageAccountName $storageAccountName -Location "local" -Type Standard_LRS
    Write-Verbose -Message "Storage account: $($storageAccount.StorageAccountName)" -Verbose

    Set-AzureRmCurrentStorageAccount -ResourceGroupName $resourceGroupName -StorageAccountName $storageAccountName | Out-Null

    # Blob storage container
    $storageContainer = New-AzureStorageContainer -Name $storageContainerName -Permission Container -ErrorAction SilentlyContinue
    $storageContainer = Get-AzureStorageContainer -Name $storageContainerName
    Write-Verbose -Message "Storage container: $($storageContainer.Name)" -Verbose

    $sourceFiles = Get-ChildItem -Path $path -Filter $filter -File -Recurse
    foreach ($sourceFile in $sourceFiles)
    {
        Write-Verbose -Message "Uploading file $sourceFile to storage container $($storageContainer.Name)" -Verbose
        $blob = Set-AzureStorageBlobContent -File $sourceFile.FullName -Container $storageContainerName -Force
        $blobs += $blob
    }

    return $blobs
}

function Remove-FilesFromBlobStorage
{
    param(
        [Parameter(Mandatory=$true, HelpMessage="Path of files to upload")]
        [string]$path,
        [Parameter(Mandatory=$true, HelpMessage="Resource group name")]
        [string]$resourceGroupName,
        [Parameter(Mandatory=$true, HelpMessage="Storage account name")]
        [string]$storageAccountName,
        [Parameter(Mandatory=$true, HelpMessage="Storage container name")]
        [string]$storageContainerName
    )
     account
    $storageAccount = Get-AzureRmStorageAccount -ResourceGroupName $resourceGroupName -StorageAccountName $storageAccountName -Location "local"
    Write-Verbose -Message "Storage account: $($storageAccount.StorageAccountName)" -Verbose

    Set-AzureRmCurrentStorageAccount -ResourceGroupName $resourceGroupName -StorageAccountName $storageAccountName

    # Blob storage container
    $storageContainer = Get-AzureStorageContainer -Name $storageContainerName
    Write-Verbose -Message "Storage container: $($storageContainer.Name)" -Verbose

    $sourceFiles = Get-ChildItem -Path $path -Filter $filter -File -Recurse
    foreach ($sourceFile in $sourceFiles)
    {
        Write-Verbose -Message "Uploading file $sourceFile to storage container $($storageContainer.Name)" -Verbose
        $blob = Remove-AzureStorageBlob -File $sourceFile.FullName -Container $storageContainerName -Force
    }
}

function Test-TemplateFile
{
     param(
        [Parameter(Mandatory=$true, HelpMessage="Template file path to be tested")]
        [string]$path
    )
    try
    {
        $json = Get-Content -Path $path -Raw | ConvertFrom-Json
    }
    catch
    {
        $err = "Cannot parse JSON template file '$path'.Verify ANSI encoding, no /*comments*/, and valid JSON syntax.`r`nException: $($_.Exception.Message)"
        throw New-Object System.ArgumentException($err, $_.Exception)
    }
}

function Register-ResourceProviderNamespace
{
    param(
        [Parameter(Mandatory=$true, HelpMessage="Provider Namespace")]
        [string]$namespace
    )
    $maxRetryCount = 60
    $currentRetryCount = 0
    $duration = 10
    $registrationSuccess = $false
    while($currentRetryCount -lt $maxRetryCount)
    {
        $resourceprovider = Get-AzureRmResourceProvider -ListAvailable | ? {$_.ProviderNamespace -eq $namespace}
        if($resourceprovider -ne $null)
        {
            if($resourceprovider.RegistrationState -ne "Registered")
            {
                Write-Verbose -Verbose -Message "Register resource provider: $namespace"
                $resourceprovider | Register-AzureRmResourceProvider -Force
            }

            if($resourceprovider.RegistrationState -eq "Registered")
            {
                Write-Verbose -Verbose -Message "Resource provider: $namespace is registered"
                $registrationSuccess = $true
                break;
            }
            else
            {
                Start-Sleep -Seconds $duration
                Write-Verbose -Verbose -Message ("Waiting for registering resource provider {0} for {1} seconds" -f ($namespace,(($currentRetryCount + 1) * $duration)))
            }
        }
        else
        {
            Start-Sleep -Seconds $duration
            Write-Warning -Message ("Cannot find resource provider {0} for {1} seconds" -f ($namespace,(($currentRetryCount + 1) * $duration)))
        }
        $currentRetryCount++
    }
    if(-not $registrationSuccess)
    {
        throw "Cannot register resource provider: $namespace"
    }
}

function Test-ResourceProviderConnection
{
    param(
        [Parameter(Mandatory=$true, HelpMessage="VM Name")]
        [string]$VmName,
        [Parameter(Mandatory=$true, HelpMessage="Azure connection port")]
        [string]$ResourceProviderPort,
        [Parameter(Mandatory=$false, HelpMessage="Max Retry Count")]
        [string]$maxRetryCount = 60,
        [Parameter(Mandatory=$false, HelpMessage="Duration")]
        [string]$duration = 60,
        [Parameter(Mandatory=$true, HelpMessage="MAS WAS machine name.  e.g., MAS-WAS01")]
        [string]$wasVMName
    )
    $currentCount = 0
    do
    {
        Invoke-Command $wasVMName { ipconfig /flushdns } | Out-Null
        $connected = Invoke-Command $wasVMName { (Test-NetConnection -ComputerName $using:VmName -Port $using:ResourceProviderPort).TcpTestSucceeded }
        if(-not $connected)
        {
            Start-Sleep -Seconds $duration
            $currentCount ++
            Write-Verbose -Verbose -Message ("Cannot Connect to {0}:{1} in {2} attempt(s)"-f $VmName, $ResourceProviderPort, $currentCount)
        }
        else
        {
            break
        }
    }
    while($currentCount -le $maxRetryCount)
}

function Get-AzureRmProviderRegistrationInfo
{
    param(
        [Parameter(Mandatory=$true, HelpMessage="Resource Provider path")]
        $azResourceProvider
    )    
    $rpInfo = [PSCustomObject]@{
        Name = $azResourceProvider.Name
        DisplayName = $azResourceProvider.Properties.DisplayName
        ManifestEndpointUri =  $azResourceProvider.Properties.ManifestEndpoint.EndpointUri
        Location = $azResourceProvider.Location
        Enabled = $azResourceProvider.Properties.Enabled
    }
    return $rpInfo
}

function Read-InputBoxDialog
{
    param(
        [Parameter(Mandatory=$true, HelpMessage="Message to be displayed on the Text box")]
        [string]$Message,
        [Parameter(Mandatory=$true, HelpMessage="Title of the text box")]
        [string]$WindowTitle,
        [Parameter(Mandatory=$false, HelpMessage="Any default Text to be filled in the text box")]
        [string]$DefaultText                
    )      
    Add-Type -AssemblyName Microsoft.VisualBasic 
    return [Microsoft.VisualBasic.Interaction]::InputBox($Message, $WindowTitle, $DefaultText)
}

function Test-Credential 
{
    param(
        [parameter(Mandatory=$true,ValueFromPipeline=$true)]
        [System.Management.Automation.PSCredential]$credential
    )
    begin {
        $context = 'Domain'
        Add-Type -assemblyname system.DirectoryServices.accountmanagement
        $DS = New-Object System.DirectoryServices.AccountManagement.PrincipalContext([System.DirectoryServices.AccountManagement.ContextType]::$context) 
    }
    process {
        $value = $DS.ValidateCredentials($credential.UserName, $credential.GetNetworkCredential().password,[System.DirectoryServices.AccountManagement.ContextOptions]::SimpleBind)
        return [bool]$value
    }
}

Function Test-VMCredential{

    param(
        [parameter(Mandatory=$true)]
        [string]$UserName,
        [parameter(Mandatory=$true)]
        [string]$Password
    )

    if($UserName -match '[^a-zA-Z0-9]' -or $UserName.length -gt 64 -or $UserName.length -lt 2)
    {
        $s = "Given Username $UserName does not meet the requirements of:" + [Environment]::NewLine + "1) Having only numbers or upper and lower case letters." + [Environment]::NewLine + "2) Length between 2 and 64 characters"
        Write-Verbose $s -Verbose
        return $false
    }
    
    $upper = [regex]"[A-Z]"
    $lower = [regex]"[a-z]"
    $number = [regex]"[0-9]"
    #Special is "none of the above"
    $special = [regex]"[^a-zA-Z0-9]"
    [bool]$return = $true
    # Check the length.
    if($Password.length -lt 8) 
    {
        $return = $false
    }

    # Check for minimum number of occurrences.
    $checks = 0
    $checks += If ($upper.Matches($Password).Count -gt 0 ) {1} Else {0}
    $checks += If ($lower.Matches($Password).Count -gt 0) {1} Else {0}
    $checks += If ($number.Matches($Password).Count -gt 0) {1} Else {0}
    $checks += If ($special.Matches($Password).Count -gt 0) {1} Else {0}
    if($checks -lt 3)
    {
        $return = $false
    }

    # Passed all checks.
    if(!$return)
    {
        $s = "Given Password does not meet the requirements of" + [Environment]::NewLine + "1) Having at least one upper case letter, one lower case letter, one number and one special character." + [Environment]::NewLine + "2) Length between 8 and 64 characters"
        Write-Verbose $s -Verbose
        return $false
    }
    $true
}
function Test-VmName
{
    param(
        [Parameter(Mandatory=$true, HelpMessage="Virtual Machine Name")]
        [string]$VmName
    )
    $valid = $true

    if($VmName.length -gt 15) 
    {
        $valid = $false
    }


    $special = [regex]"[^a-zA-Z0-9]"
    if ($special.Matches($VmName).Count -gt 0)
    {
        $valid = $false
    }
    
    if(!$valid)
    {
        $s = "Given VM Name does not meet the requirements of" + [Environment]::NewLine + "1) Having only alphanumeric characters." + [Environment]::NewLine + "2) Length between 1 and 15 characters"
        Write-Verbose $s -Verbose
        return $false
    }
    return $true

}

function Test-DependencyFilesLocalPath
{
    param(
        [Parameter(Mandatory=$true, HelpMessage="Path to dependency files")]
        [string]$Path
    )
       
    if (!$Path) 
    {
        return $true
    }
    $valid = Test-Path $Path
    if(!$valid)
    {
        $s = "The provided local path to the dependency files does not exist."
        Write-Verbose $s -Verbose
        return $false
    }
    return $true
}

function Test-ResourceGroup
{
    param(
        [Parameter(Mandatory=$true, HelpMessage="Name of Resource Group")]
        [string]$Name
    )

    $valid = $true   
    if($Name.length -gt 80) 
    {
        $valid = $false
    }

    $invalid = [regex]"[^a-zA-Z0-9().\-_]"
    if ($invalid.Matches($Name).Count -gt 0)
    {
        $valid = $false
    }

    $lastChar = [regex]"[_.-]"
    if ($lastChar.Matches($Name.Substring($Name.Length -1, 1)).Count -gt 0)
    {
        $valid = $false
    }
    if ($lastChar.Matches($Name.Substring(0, 1)).Count -gt 0)
    {
        $valid = $false
    }

    if(!$valid)
    {
        $s = "The Resource Group Name:  1) can only have alphanumeric characters, hyphens, periods, and underscores. Cannot begin or end with special characters. 2) Must be less than 80 characters long."
        Write-Verbose $s -Verbose
        return $false
    }
    return $true
}
