﻿# Copyright (c) CloudAssert LLC. All rights reserved.
# Publish-GalleryPackages.ps1 5.11.251.0 2016-11-04 10:37:08

#requires -Version 4.0
#requires -Module AzureRM
#requires -Module AzureStack

<#
.SYNOPSIS
   Publishes Azure Package (*.azpkg) files to Marketplace (Gallery) in Azure Stack environment.
#>
[CmdletBinding()]
param
(
    [Parameter(Mandatory=$true, HelpMessage="Azure Active Directory tenant directory name.")]
    [string]$AadTenantDirectoryName,

    [Parameter(Mandatory=$true, HelpMessage="Azure Stack Service Admin account Username.")]
    [string]$azUserName,

    [Parameter(Mandatory=$true, HelpMessage="Azure Stack Service Admin account Password.")]
    [string]$azPassword,

    [Parameter(Mandatory=$true, HelpMessage="Root path of folder where all *.azpkg files are present.")]
    [string]$galleryPackagesRootPath,

    [Parameter(Mandatory=$false, HelpMessage="The GUID used to identify this deployment for telemetry purposes.")]
    [guid]$DeploymentId,
    
    [Parameter(Mandatory=$true, HelpMessage="Azure Stack Admin ARM Endpoint URL")]
    [string]$AdminArmEndpoint,
    
    [Parameter(Mandatory=$true, HelpMessage="Azure Stack Portal Endpoint Suffix.  e.g., local.azurestack.external")]
    [string]$PortalEndpointSuffix,
    
    [Parameter(Mandatory=$true, HelpMessage="Cloud Assert Billing RP endpoint URL.  e.g., https://billingrp.azurestack.local:13900")]
    [string]$CABillingRpEndpoint
)

Import-Module -Name AzureRM -RequiredVersion 1.2.10 -ErrorAction Stop -Verbose:$false
Import-Module -Name AzureStack -RequiredVersion 1.2.10 -ErrorAction Stop -Verbose:$false
Import-Module -Name "$PSScriptRoot\Common.psm1" -ErrorAction Stop -Verbose:$false


#-----------------------------------------------------------------------------------------------------------------------
# Main

$stopwatch = [System.Diagnostics.Stopwatch]::StartNew()

[System.Collections.ArrayList]$logMessages = @()
$scriptName = $MyInvocation.MyCommand.Name
$startTime = [System.DateTimeOffset]::Now
Write-Verbose -Message "Executing Script $scriptName at $startTime" -Verbose
Write-Verbose -Message "Using Parameters: " -Verbose
Write-Verbose -Message $($PSBoundParameters | ConvertTo-Json)  -Verbose

# Telemetry
$deploymentGUID = $DeploymentId # (Get-WmiObject Win32_ComputerSystemProduct).UUID
$startTime = [System.DateTimeOffset]::Now
#Trace-DeploymentInternal -DeploymentGUID $deploymentGUID -Step "Publish-GalleryPackages.ps1 5.11.251.0" -StartTime $startTime -Status 'Success' -LogMessage "Publish CloudAssert.CABilling provider gallery packages started."

$secPassword = ConvertTo-SecureString $azPassword -AsPlainText -Force
$azCredential = New-Object System.Management.Automation.PSCredential($azUserName, $secPassword)

$azEnvironment = Get-AzureStackEnvironment -AadTenantDirectoryName $AadTenantDirectoryName -EnvironmentName "AzureStack" -AdminArmEndpoint $AdminArmEndpoint
$azAccount = Initialize-AzureStackEnvironment -AadTenantDirectoryName $AadTenantDirectoryName -EnvironmentName "AzureStack" -azCredential $azCredential -AdminArmEndpoint $AdminArmEndpoint

# NOTE: Select default subscription if the account contains multiple subscriptions.
$azSubscription = Get-AzureRmSubscription -SubscriptionName "Default Provider Subscription" -ErrorAction Stop
$azSubscription | Select-AzureRmSubscription | Out-Null
Write-Verbose -Message "Using subscription: '$($azSubscription.SubscriptionName)'" -Verbose

# Dangerous and Destructive operation: Delete existing
$existingGalleryItems = Get-AzureRMGalleryItem -Verbose
if ($existingGalleryItems -ne $null) {
	foreach ($galleryItem in $existingGalleryItems) {
		$name = $galleryItem.Name
		if ($name -notlike '*CloudAssert.Csp*') { 
			continue; # Dangerous to 'not' skip; Not ours
		}
		try {
			Write-Verbose -Message "Removing Azure Stack Gallery item $name" -Verbose
			Remove-AzureRMgalleryitem -Name $name -Verbose
		} catch [Exception] {
			Write-Error "Failed to remove Azure Stack Gallery item $name . Error $_"
		}
	}
}

Write-Verbose -Message "Looking for *.azpkg files under '$galleryPackagesRootPath'..." -verbose
$azpkgFiles = Get-ChildItem -Path $galleryPackagesRootPath -Filter "*.azpkg" -File -Recurse
$azpkgFiles | Format-Table Name,Length,LastWriteTime -AutoSize

foreach ($azpkgFile in $azpkgFiles)
{
    $name = [System.IO.Path]::GetFileNameWithoutExtension($azpkgFile.Name)
    try {
        #Write-Verbose -Message "Removing file $name from  Azure Gallery" -Verbose
        #Remove-AzureRMgalleryitem -Name $name -SubscriptionId $azSubscription.SubscriptionId -AdminUri $azEnvironment.ResourceManagerUrl -ApiVersion "2015-04-01" -Verbose
    } catch [Exception] {
        Write-Error "Failed to remove package $name from Azure Gallery with an error $_"
    }
    try {
        $azpkgUri = $CABillingRpEndpoint + "/Gallery/Packages/" + $azpkgFile.Name
        $message = "Adding file $azpkgUri to Azure Gallery..."
        Write-Verbose -Message $message -Verbose
        $logMessages.Add($message)
        
        $response = Add-AzureRMGalleryItem -GalleryItemUri $azpkgUri -Verbose
        $resposeStr = ConvertTo-Json $response
        $logMessages.Add($resposeStr)
    } catch [Exception] {
        Write-Error "Failed to add file $azpkgUri to Azure Gallery with an error $_"
    }
}

# Telemetry
$endTime = [System.DateTimeOffset]::Now
#Trace-DeploymentInternal -DeploymentGUID $deploymentGUID -Step "Publish-GalleryPackages.ps1 5.11.251.0" -StartTime $startTime -EndTime $endTime -Status 'Success' -LogMessage "Publish CloudAssert.CABilling provider gallery packages completed."
return ConvertTo-Json -Depth 10 @{
    IsSuccess = $true
    LogMessages = $logMessages
    ErrorMessages = @()
}
